from django.contrib import admin
from telegrambot.models import Message, Chat, Update, User

admin.site.register(Message)
admin.site.register(Chat)
admin.site.register(User)
admin.site.register(Update)


from django.conf.urls import url
from django.views.decorators.csrf import csrf_exempt
from telegrambot import views


urlpatterns = [
    url(r'^webhook/(?P<token>[-_:a-zA-Z0-9]+)/$', csrf_exempt(views.WebhookView.as_view()), name='webhook'),
]


# coding=utf-8
from factory import Sequence, SubFactory, Factory
from telegram import Update, Message, User, Chat
from django.utils import timezone
from factory.fuzzy import FuzzyText


class UserFactory(Factory):
    class Meta:
        model = User
    id = Sequence(lambda n: n+1)
    first_name = Sequence(lambda n: 'first_name_%d' % n)
    last_name = Sequence(lambda n: 'last_name_%d' % n)
    username = Sequence(lambda n: 'username_%d' % n)

class ChatFactory(Factory):
    class Meta:
        model = Chat
    id = Sequence(lambda n: n+1)
    type = "private"
    title = Sequence(lambda n: 'title_%d' % n)
    username = Sequence(lambda n: 'username_%d' % n)
    first_name = Sequence(lambda n: 'first_name_%d' % n)
    last_name = Sequence(lambda n: 'last_name_%d' % n)

class MessageFactory(Factory):
    class Meta:
        model = Message
    message_id = Sequence(lambda n: n+1)
    from_user = SubFactory(UserFactory)
    date = timezone.now()
    chat = SubFactory(ChatFactory)
    text = FuzzyText()    

class UpdateFactory(Factory):
    class Meta:
        model = Update
    update_id = Sequence(lambda n: n+1)
    message = SubFactory(MessageFactory)


from telegrambot.generic.base import TemplateCommandView  # noqa
from telegrambot.generic.compound import ListDetailCommandView  # noqa
from telegrambot.generic.detail import DetailCommandView  # noqa
from telegrambot.generic.list import ListCommandView  # noqa
from telegrambot.generic.responses import (TextResponse, KeyboardResponse)  # noqa


from rest_framework.views import APIView
from telegrambot.serializers import UpdateSerializer
from rest_framework.response import Response
from rest_framework import status
from telegram import Update
import logging
from telegrambot.bot import Bot

logger = logging.getLogger(__name__)

class WebhookView(APIView):    
    command_patterns = None

    def __init__(self, *args, **kwargs):
        super(WebhookView, self).__init__(*args, **kwargs)        
        self.bot = Bot()
    
    def post(self, request, token):
        serializer = UpdateSerializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            try:
                self.bot.process_update(Update.de_json(request.data))
            except:
                return Response(serializer.errors, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
            else:
                return Response(serializer.data, status=status.HTTP_200_OK)
        logger.info("Validation error: %s from message %s" % (serializer.errors, request.data))
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


from django.template import RequestContext, TemplateDoesNotExist
from django.template.loader import get_template
from telegram import ReplyKeyboardMarkup, ReplyKeyboardHide
import ast
import logging
from django.http.request import HttpRequest

logger = logging.getLogger(__name__)

class TemplateResponse(object):
    
    def __init__(self, template_name, ctx=None):
        self.template_name = template_name
        if ctx is None:
            self.ctx = {}
        else:
            self.ctx = ctx
        
    def render(self):
        if not self.template_name:
            return None
        try:
            logger.debug("Template name: %s" % self.template_name)
            template = get_template(self.template_name)      
        except TemplateDoesNotExist:
            logger.debug("Template not found: %s" % self.template_name)
            return None
        # TODO: Avoid using a null HttRequest to context processors
        ctx = RequestContext(HttpRequest(), self.ctx)
        return template.render(ctx)
    
class TextResponse(TemplateResponse):

    def __init__(self, template_text, ctx=None):
        super(TextResponse, self).__init__(template_text, ctx)
        
class KeyboardResponse(TemplateResponse):
    
    def __init__(self, template_keyboard, ctx=None):
        super(KeyboardResponse, self).__init__(template_keyboard, ctx)
        
    def render(self):
        keyboard = super(KeyboardResponse, self).render()
        if keyboard:
            keyboard = ast.literal_eval(keyboard)
            keyboard = ReplyKeyboardMarkup(keyboard, resize_keyboard=True)
        else:
            keyboard = ReplyKeyboardHide()
        return keyboard


from django.core.management.base import BaseCommand, CommandError
import logging
from django.core.urlresolvers import reverse
from telegrambot import conf
from telegram import Bot


class Command(BaseCommand):
    help = "Set webhook url to Telegram"
    
    def handle(self, *args, **options):
        logger = self._get_logger()
        if not conf.TELEGRAM_BOT_TOKEN:
            raise CommandError("Telegram bot requires Telegram token.")
        token = conf.TELEGRAM_BOT_TOKEN
        logger.info("Setting webhook for bot %s ", token)
        bot = Bot(token=token)
        webhook = reverse('telegrambot:webhook', kwargs={'token': token})        
        from django.contrib.sites.models import Site
        current_site = Site.objects.get_current()
        url = 'https://' + current_site.domain + webhook
        logger.info("setWebhook url %s ", url)
        bot.setWebhook(url)
        logger.info("Success: Webhook url %s for token %s set" % (url, token))

    def _get_logger(self):
        logger = logging.getLogger(__file__)
        stream = logging.StreamHandler(self.stdout)
        logger.addHandler(stream)
        logger.setLevel(logging.DEBUG)
        return logger


# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-01-21 15:46
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Chat',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
                ('type', models.CharField(choices=[(b'private', 'Private'), (b'group', 'Group'), (b'supergroup', 'Supergroup'), (b'channel', 'Channel')], max_length=255)),
                ('title', models.CharField(blank=True, max_length=255, null=True)),
                ('username', models.CharField(blank=True, max_length=255, null=True)),
                ('first_name', models.CharField(blank=True, max_length=255, null=True)),
                ('last_name', models.CharField(blank=True, max_length=255, null=True)),
            ],
            options={
                'verbose_name': 'Chat',
                'verbose_name_plural': 'Chats',
            },
        ),
        migrations.CreateModel(
            name='Message',
            fields=[
                ('message_id', models.BigIntegerField(primary_key=True, serialize=False, verbose_name='Id')),
                ('date', models.DateTimeField(verbose_name='Date')),
                ('text', models.TextField(blank=True, null=True, verbose_name='Text')),
                ('chat', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='messages', to='telegrambot.Chat', verbose_name='Chat')),
            ],
            options={
                'ordering': ['-date'],
                'verbose_name': 'Message',
                'verbose_name_plural': 'Messages',
            },
        ),
        migrations.CreateModel(
            name='Update',
            fields=[
                ('update_id', models.BigIntegerField(primary_key=True, serialize=False, verbose_name='Id')),
                ('message', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='updates', to='telegrambot.Message', verbose_name='Message')),
            ],
            options={
                'verbose_name': 'Update',
                'verbose_name_plural': 'Updates',
            },
        ),
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
                ('first_name', models.CharField(max_length=255, verbose_name='First name')),
                ('last_name', models.CharField(blank=True, max_length=255, null=True, verbose_name='Last name')),
                ('username', models.CharField(blank=True, max_length=255, null=True, verbose_name='User name')),
            ],
            options={
                'verbose_name': 'User',
                'verbose_name_plural': 'Users',
            },
        ),
        migrations.AddField(
            model_name='message',
            name='forward_from',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='forwarded_from', to='telegrambot.User', verbose_name='Forward from'),
        ),
        migrations.AddField(
            model_name='message',
            name='from_user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='messages', to='telegrambot.User', verbose_name='User'),
        ),
    ]



# -*- coding: utf-8 -*-
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

@python_2_unicode_compatible
class User(models.Model):
    id = models.BigIntegerField(primary_key=True)
    first_name = models.CharField(_('First name'), max_length=255)
    last_name = models.CharField(_('Last name'), max_length=255, blank=True, null=True)
    username = models.CharField(_('User name'), max_length=255, blank=True, null=True)

    class Meta:
        verbose_name = _('User')
        verbose_name_plural = _('Users')

    def __str__(self):
        return "%s" % self.first_name

@python_2_unicode_compatible
class Chat(models.Model):

    PRIVATE, GROUP, SUPERGROUP, CHANNEL = 'private', 'group', 'supergroup', 'channel'

    TYPE_CHOICES = (
        (PRIVATE, _('Private')),
        (GROUP, _('Group')),
        (SUPERGROUP, _('Supergroup')),
        (CHANNEL, _('Channel')),
    )

    id = models.BigIntegerField(primary_key=True)
    type = models.CharField(max_length=255, choices=TYPE_CHOICES)
    title = models.CharField(max_length=255, null=True, blank=True)
    username = models.CharField(max_length=255, null=True, blank=True)
    first_name = models.CharField(max_length=255, null=True, blank=True)
    last_name = models.CharField(max_length=255, null=True, blank=True)

    class Meta:
        verbose_name = _('Chat')
        verbose_name_plural = _('Chats')

    def __str__(self):
        return "%s" % (self.title or self.username)

@python_2_unicode_compatible
class Message(models.Model):

    message_id = models.BigIntegerField(_('Id'), primary_key=True)
    from_user = models.ForeignKey(User, related_name='messages', verbose_name=_("User"))
    date = models.DateTimeField(_('Date'))
    chat = models.ForeignKey(Chat, related_name='messages', verbose_name=_("Chat"))
    forward_from = models.ForeignKey(User, null=True, blank=True, related_name='forwarded_from',
                                     verbose_name=_("Forward from"))
    text = models.TextField(null=True, blank=True, verbose_name=_("Text"))
    #  TODO: complete fields with all message fields

    class Meta:
        verbose_name = 'Message'
        verbose_name_plural = 'Messages'
        ordering = ['-date', ]

    def __str__(self):
        return "(%s,%s)" % (self.from_user, self.text or '(no text)')
    
class Update(models.Model):
    
    update_id = models.BigIntegerField(_('Id'), primary_key=True)
    message = models.ForeignKey(Message, null=True, blank=True, verbose_name=_('Message'), 
                                related_name="updates")
    
    class Meta:
        verbose_name = 'Update'
        verbose_name_plural = 'Updates'
    
    def __str__(self):
        return "%s" % self.update_id    



from telegram import Updater
import importlib
from django.core.exceptions import ImproperlyConfigured
from telegrambot import conf

class Bot(object):
    
    def __init__(self):
        if not conf.TELEGRAM_BOT_TOKEN:
            raise ImproperlyConfigured("Telegram bot requires Telegram token.")
        if not conf.TELEGRAM_BOT_HANDLERS_CONF:
            bothandlers = []
        else:
            bothandlers = importlib.import_module(conf.TELEGRAM_BOT_HANDLERS_CONF).bothandlers
        self.updater = Updater(conf.TELEGRAM_BOT_TOKEN)
        #  on different bot_handlers - answer in Telegram
        for handler in bothandlers:
            handler.add_to_dispatcher(self.updater.dispatcher)
   
    def process_update(self, update):
        self.updater.dispatcher.processUpdate(update)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.test import TestCase
from telegrambot.models import Update
from telegrambot.test import factories
from django.core.urlresolvers import reverse
from rest_framework import status
from telegram.replykeyboardhide import ReplyKeyboardHide
from telegrambot import conf
from django.conf import settings
try:
    from unittest import mock
except ImportError:
    import mock  # noqa


class BaseTestBot(TestCase):    

    def setUp(self):
        self.webhook_url = reverse('telegrambot:webhook', kwargs={'token': conf.TELEGRAM_BOT_TOKEN})
        self.update = factories.UpdateFactory()
        self.kwargs = {'content_type': 'application/json', }
        
    def tearDown(self):
        if not conf.TELEGRAM_BOT_TOKEN:
            conf.TELEGRAM_BOT_TOKEN = getattr(settings, 'TELEGRAM_BOT_TOKEN', None)
        
    def assertUser(self, model_user, user):
        self.assertEqual(model_user.id, user.id)
        self.assertEqual(model_user.first_name, user.first_name)
        self.assertEqual(model_user.last_name, user.last_name)
        self.assertEqual(model_user.username, user.username)
        
    def assertChat(self, model_chat, chat):        
        self.assertEqual(model_chat.id, chat.id)
        self.assertEqual(model_chat.type, chat.type)
        self.assertEqual(model_chat.title, chat.title)
        self.assertEqual(model_chat.username, chat.username)
        self.assertEqual(model_chat.first_name, chat.first_name)
        self.assertEqual(model_chat.last_name, chat.last_name)
        
    def assertMessage(self, model_message, message):        
        self.assertEqual(model_message.message_id, message.message_id)
        self.assertUser(model_message.from_user, message.from_user)
        self.assertChat(model_message.chat, message.chat)
        #  TODO: problems with UTCs
        #  self.assertEqual(model_message.date, message.date)
        self.assertEqual(model_message.text, message.text)
        
    def assertUpdate(self, model_update, update):
        self.assertEqual(model_update.update_id, update.update_id)
        self.assertMessage(model_update.message, update.message)
        
    def assertInKeyboard(self, button, keyboard):
        found = False
        for line in keyboard:
            if button in line:
                found = True
                break
        self.assertTrue(found)
        
    def assertBotResponse(self, mock_send, command):
        args, kwargs = mock_send.call_args
        self.assertEqual(1, mock_send.call_count)
        self.assertEqual(kwargs['chat_id'], self.update.message.chat.id)
        self.assertEqual(kwargs['parse_mode'], command['out']['parse_mode'])
        if not command['out']['reply_markup']:
            self.assertTrue(isinstance(kwargs['reply_markup'], ReplyKeyboardHide))
        else:
            self.assertInKeyboard(command['out']['reply_markup'], kwargs['reply_markup'].keyboard)
                
        self.assertIn(command['out']['text'], kwargs['text'].decode('utf-8'))
        
    def _test_message_ok(self, action, update=None, number=1):
        if not update:
            update = self.update
        with mock.patch("telegram.bot.Bot.sendMessage", callable=mock.MagicMock()) as mock_send:
            if 'in' in action:
                update.message.text = action['in']
            response = self.client.post(self.webhook_url, update.to_json(), **self.kwargs)
            #  Check response 200 OK
            self.assertEqual(response.status_code, status.HTTP_200_OK)
            #  Check  
            self.assertBotResponse(mock_send, action)
            self.assertEqual(number, Update.objects.count())
            self.assertUpdate(Update.objects.get(update_id=update.update_id), update)


from telegrambot.generic.base import TemplateCommandView
from django.core.exceptions import ImproperlyConfigured
from django.db.models.query import QuerySet
from django.utils import six

class ListCommandView(TemplateCommandView):
    queryset = None
    context_object_name = None
    model = None
    ordering = None
    
    def get_queryset(self):
        if self.queryset is not None:
            queryset = self.queryset
            if isinstance(queryset, QuerySet):
                queryset = queryset.all()
        elif self.model is not None:
            queryset = self.model._default_manager.all()
        else:
            raise ImproperlyConfigured(
                "%(cls)s is missing a QuerySet. Define "
                "%(cls)s.model, %(cls)s.queryset, or override "
                "%(cls)s.get_queryset()." % {
                    'cls': self.__class__.__name__
                }
            )
        ordering = self.get_ordering()
        if ordering:
            if isinstance(ordering, six.string_types):
                ordering = (ordering,)
            queryset = queryset.order_by(*ordering)
        return queryset
    
    def get_ordering(self):
        """
        Return the field or fields to use for ordering the queryset.
        """
        return self.ordering
    
    def get_context(self, update, **kwargs):
        object_list = self.get_queryset()
        context = {'object_list': object_list}
        if self.context_object_name:
            context[self.context_object_name] = object_list
        return context


from telegrambot.generic.responses import TextResponse, KeyboardResponse
from telegram import ParseMode
import sys
import traceback
import logging

logger = logging.getLogger(__name__)


class TemplateCommandView(object):
    template_text = None
    template_keyboard = None    
    
    def get_context(self, update, **kwargs):
        return None    

    def handle(self, bot, update, **kwargs):
        try:
            ctx = self.get_context(update, **kwargs)
            text = TextResponse(self.template_text, ctx).render()
            keyboard = KeyboardResponse(self.template_keyboard, ctx).render()        
#             logger.debug("Text:" + str(text.encode('utf-8')))
#             logger.debug("Keyboard:" + str(keyboard))
            if text:
                bot.sendMessage(chat_id=update.message.chat_id, text=text.encode('utf-8'), reply_markup=keyboard, parse_mode=ParseMode.MARKDOWN)
            else:
                logger.info("No text response for update %s" % str(update))
        except:
            exc_info = sys.exc_info()
            traceback.print_exception(*exc_info)
            raise

    @classmethod
    def as_command_view(cls, **initkwargs):
        def view(bot, update, groupdict=None):
            self = cls(**initkwargs)
            return self.handle(bot, update, pattern=groupdict)
        return view




class BotHandler(object):
    
    def __init__(self, callback):
        if not callable(callback):
            raise TypeError('view must be a callable')
        self.callback = callback
    
    def add_to_dispatcher(self, dispatcher):
        raise NotImplementedError
    
class RegexHandler(BotHandler):
    
    def __init__(self, pattern, callback):
        super(RegexHandler, self).__init__(callback)
        self.pattern = pattern
        
    def add_to_dispatcher(self, dispatcher):
        dispatcher.addTelegramRegexHandler(self.pattern, self.callback)
        
class CommandHandler(BotHandler):
    
    def __init__(self, command, callback):
        super(CommandHandler, self).__init__(callback)
        self.command = command
        
    def add_to_dispatcher(self, dispatcher):
        dispatcher.addTelegramCommandHandler(self.command, self.callback)
        
class UnknownCommandHandler(BotHandler):
    
    def add_to_dispatcher(self, dispatcher):
        dispatcher.addUnknownTelegramCommandHandler(self.callback)
        
class MessageHandler(BotHandler):
    
    def add_to_dispatcher(self, dispatcher):
        dispatcher.addTelegramMessageHandler(self.callback)

def regex(pattern, view):
    return RegexHandler(pattern, view)
        
def command(command, view):
    return CommandHandler(command, view)

def unknown_command(view):
    return UnknownCommandHandler(view)

def message(view):
    return MessageHandler(view)


from telegrambot.generic.base import TemplateCommandView

class ListDetailCommandView(TemplateCommandView):
    list_view_class = None
    detail_view_class = None
    
    @classmethod
    def as_command_view(cls, **initkwargs):
        def view(bot, update, **kwargs):
            command_args = update.message.text.split(' ')
            if len(command_args) > 1:
                self = cls.detail_view_class(command_args[1])
            else:
                self = cls.list_view_class()
            return self.handle(bot, update, **kwargs)
        return view


from telegrambot.generic.base import TemplateCommandView
from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist,\
    FieldError

class DetailCommandView(TemplateCommandView):
    model = None
    queryset = None
    context_object_name = None
    slug_field = 'slug'    
    
    def __init__(self, slug=None):
        self.slug = slug
    
    def get_slug_field(self, **kwargs):
        """
        Get the name of a slug field to be used to look up by slug.
        """
        return self.slug_field
    
    def get_queryset(self):
        """
        Return the `QuerySet` that will be used to look up the object.
        Note that this method is called by the default implementation of
        `get_object` and may not be called if `get_object` is overridden.
        """
        if self.queryset is None:
            if self.model:
                return self.model._default_manager.all()
            else:
                raise ImproperlyConfigured(
                    "%(cls)s is missing a QuerySet. Define "
                    "%(cls)s.model, %(cls)s.queryset, or override "
                    "%(cls)s.get_queryset()." % {
                        'cls': self.__class__.__name__
                    }
                )
        return self.queryset.all()
    
    def get_slug(self, **kwargs):
        return self.slug
    
    def get_context(self, update, **kwargs):
        queryset = self.get_queryset()
        if not self.slug_field: 
            raise AttributeError("Generic detail view %s must be called with "
                                 "a slug."
                                 % self.__class__.__name__)
        slug_field = self.get_slug_field(**kwargs)
        slug = self.get_slug(**kwargs)
        if slug:
            try:
                object = queryset.get(**{slug_field: slug})
            except FieldError:
                raise FieldError("Field %s not in valid. Review slug_field" % slug_field)
            except ObjectDoesNotExist:
                object = None
        else: 
            object = None
        context = {'context_object_name': object}
        if self.context_object_name:
            context[self.context_object_name] = object
        return context


from rest_framework import serializers
from telegrambot.models import User, Chat, Message, Update
from datetime import datetime
import time

class UserSerializer(serializers.HyperlinkedModelSerializer):
    id = serializers.IntegerField()

    class Meta:
        model = User
        fields = ('id', 'first_name', 'last_name', 'username')
        
class ChatSerializer(serializers.HyperlinkedModelSerializer):
    id = serializers.IntegerField()

    class Meta:
        model = Chat
        fields = ('id', 'type', 'title', 'username', 'first_name', 'last_name')
        
class TimestampField(serializers.Field):

    def to_internal_value(self, data):
        return datetime.fromtimestamp(data)
    
    def to_representation(self, value):
        return int(time.mktime(value.timetuple()))

        
class MessageSerializer(serializers.HyperlinkedModelSerializer):
    message_id = serializers.IntegerField()
    # reserved word field 'from' changed dynamically
    from_ = UserSerializer(many=False, source="from_user")
    chat = ChatSerializer(many=False)
    date = TimestampField()
    
    def __init__(self, *args, **kwargs):
        super(MessageSerializer, self).__init__(*args, **kwargs)
        self.fields['from'] = self.fields['from_']
        del self.fields['from_']

    class Meta:
        model = Message
        fields = ('message_id', 'from_', 'date', 'chat', 'text')
        
class UpdateSerializer(serializers.HyperlinkedModelSerializer):
    update_id = serializers.IntegerField()
    message = MessageSerializer(many=False)
    
    class Meta:
        model = Update
        fields = ('update_id', 'message')
        
    def create(self, validated_data):
        user, _ = User.objects.get_or_create(**validated_data['message']['from_user'])
        
        chat, _ = Chat.objects.get_or_create(**validated_data['message']['chat'])
        
        message, _ = Message.objects.get_or_create(message_id=validated_data['message']['message_id'],
                                                   from_user=user,
                                                   date=validated_data['message']['date'],
                                                   chat=chat,
                                                   text=validated_data['message']['text'])
        update, _ = Update.objects.get_or_create(update_id=validated_data['update_id'],
                                                 message=message)

        return update


from django import template
from django.http import QueryDict
register = template.Library()

@register.filter(name='keyboard_field')
def keyboard_field(value, args=None):
    """
    Format keyboard /command field. 
    """
    qs = QueryDict(args)
    per_line = qs.get('per_line', 1)
    field = qs.get("field", "slug")
    command = qs.get("command")
    convert = lambda element: "/" + command + " " + str(getattr(element, field))
    group = lambda flat, size: [flat[i:i+size] for i in range(0, len(flat), size)]
    grouped = group(value, int(per_line))
    new_list = []
    for line in grouped:
        new_list.append([convert(e) for e in line])     
    return str(new_list).encode('utf-8')